#!/bin/ksh
# Licensed Materials - Property of IBM
# Restricted Materials of IBM

# Copyright IBM Corp. 2006 All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
# Release Name: omdomino_dev
#
# Description: Start up the Domino/Intelliwatch CPC(kgbclient).
#              Restart the CPC when the Domino Server has
#              shutdown.
#
# Parameters:
#   CANDLEHOME, KGB_NOTESINIPATH, and KGB_INSTANCE_NAME
#   set in the environment
#
# Returns:
#    None
#======================================================
#

# 2010/5/10: Added the fix for defect IZ76599, IZ76716 and internal defect 132593,
# which added the checking for kgbagent running during the loops to avoid the script/kgbclient running without kgbagent, 
# which will cause the agent instance can't be started nor stopped for ever except re-installing.

#determine whether kgbagent is running, return 0: running; 1:not running.
kgbagent_running()
{
    # takes process id to search for
    set -x
    kgbpid=$1
    #srun=`ps -ef|awk '{if (detect==$2) print $2}' detect=$kgbpid`
    ps -ef|awk '{print $2}' |grep $kgbpid|grep -v "grep"|while read line
    do
        echo "`date` : kgbpid: $kgbpid, kgbfound=$line" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        if [ $line = $kgbpid ]; then
          return 0
        fi
    done
    return 1
}
#check whether the process is running according to id
procid_running()
{
    pid=$1
    echo "`date` : procid check: $pid" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    kgbagent_running $pid
    return $?
}
    
#Check whether the server name is a local ip.
#return 0: is local ip. return 1: not local ip
is_localip()
{
    ipjudge=$1
    addr=""
    keyword=""
    echo "`date` :servername:$ipjudge. path:$PATH.">> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    PATH=$PATH:/usr/sbin:/etc:/sbin
    export PATH
    if [[ "$binArch" = sol+([0-9]) ]]; then
       keyword="inet"
    elif [[ "$binArch" = aix+([0-9]) ]]; then
       keyword="inet"
    else
       keyword="inet addr:"
    fi
    if [[ "$binArch" = sol+([0-9]) ]]; then
        #use nslookup for solaris because of the zones
        hostparsed=0
        hostname|while read linex
        do
            nslookup $linex|while read liney
            do
                echo "`date` :nslookup:$liney.">> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
                nm=`echo $liney|grep "Name:"`
                if [ -n "$nm" ]; then
                    hostparsed=1
                elif [ $hostparsed = 1 ]; then
                    nm=`echo $liney|awk '{print $2}'`
                    echo "`date` :host ip:$nm.">> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
                    if [ "$nm" = "$ipjudge" ]; then
                        return 0
                    fi
                fi
            done
        done
    else
        echo "`date` : ifconfig:\n`ifconfig -a`" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        ifconfig -a|while read line
        do
            addr=`echo $line|grep "$keyword"`
            #not null
            if [ -n "$addr" ]; then
                addr=`echo $addr|sed "s/$keyword//g"|awk  '{print $1}'`
                echo "`date` : line:$line, addr parsed:$addr" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
                if [ "$addr" = "$ipjudge" ]; then
                    return 0
                fi
            fi
        done
    fi
    return 1
}

# turn on tracing for log file
set -x

# save the parent pid
KGBAGENT_PID=$PPID

# make sure CANDLEHOME exists
if [ ! -d "$CANDLEHOME" ]; then
  echo "ERROR: CANDLEHOME was not found, exiting..."
  exit
fi

# make sure the instance name exists
if [ -z "$KGB_INSTANCE_NAME" ]; then
  echo "ERROR: KGB_INSTANCE_NAME was not found, exiting..."
  exit
fi

# truncate log file and modify permissions
cat /dev/null > $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
chmod 666 $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
echo "`date` : kgbstart.sh called for instance $KGB_INSTANCE_NAME" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log

# make sure the notes.ini path exists
KGB_NOTESINIPATH=`echo $KGB_NOTESINIPATH | sed -e "s/notes.ini//g"`
echo "`date` : notesini:$KGB_NOTESINIPATH" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
if [ ! -d "$KGB_NOTESINIPATH" ]; then
  echo "`date` : KGB_NOTESINIPATH : $KGB_NOTESINIPATH/notes.ini was not found, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  exit
fi

# setup path to the machine architecture
# source dynarch.shl in a subshell so it does not modify the environment
binArch=`( . $CANDLEHOME/bin/dynarch.shl $CANDLEHOME/registry/archdsc.tbl; setBinLibArchs $cArch gb; echo $binArch )`

# export as KGB_BINARCH as BSS1_genenv gets confused on binArch
KGB_BINARCH=$binArch
export KGB_BINARCH

# add for APAR#36147 and defect 90734
chmod 777 $CANDLEHOME/$binArch/gb/bin/kgbstart.sh

# determine the Lotus Domino base directory
LOTUSBASEDIR=`cat $KGB_NOTESINIPATH/notes.ini | grep -i NotesProgram | awk -F= {'print $2'}`
while true
do
  if [ -f $LOTUSBASEDIR/bin/server ]; then
    break
  else
    LOTUSBASEDIR=`dirname $LOTUSBASEDIR`
    if [ "$LOTUSBASEDIR" = "/" -o "$LOTUSBASEDIR" = "." ]; then
      break
    fi
  fi
done
if [ ! -f $LOTUSBASEDIR/bin/server ]; then
  echo "`date` : Using the NotesProgram keyword in $KGB_NOTESINIPATH/notes.ini to find the Lotus base directory, ${LOTUSBASEDIR} directory was not found, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  exit
fi

# check the Lotus Domino user id was used to start the agent
LOTUSOWNER=`ls -l ${KGB_NOTESINIPATH}/notes.ini | awk '{print $3}'`
if [ -z "$LOTUSOWNER" ]; then
  echo "`date` : Could not determine the Domino user id, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  exit
fi
if [[ "$binArch" = sol+([0-9]) ]]
then
  /usr/xpg4/bin/id -un | grep "^$LOTUSOWNER" >/dev/null 2>&1
else
  id -un | grep "^$LOTUSOWNER" >/dev/null 2>&1
fi
if [ $? -ne 0 ]; then
  ROOTUSER=yes
fi
echo "`date` :binArch:$binArch" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
# setup environement based on architecture
if [[ "$binArch" = aix+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/ibmpow:$LOTUSBASEDIR/notes/latest/ibmpow/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/ibmpow
  export Notes_ExecDirectory
  LIBPATH=$LIBPATH:$LOTUSBASEDIR/notes/latest/ibmpow
  export LIBPATH
  echo "`date` :LOTUSBASEDIR:$LOTUSBASEDIR" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
fi
if [[ "$binArch" = sol+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/sunspa:$LOTUSBASEDIR/notes/latest/sunspa/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/sunspa
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/sunspa
  export LD_LIBRARY_PATH
fi
if [[ "$binArch" = li+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/linux:$LOTUSBASEDIR/notes/latest/linux/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/linux
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/linux
  export LD_LIBRARY_PATH
fi
if [[ "$binArch" = lx+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/linux:$LOTUSBASEDIR/notes/latest/linux/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/linux
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/linux
  export LD_LIBRARY_PATH
fi
if [[ "$binArch" = ls+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/zlinux:$LOTUSBASEDIR/notes/latest/zlinux/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/zlinux
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/zlinux
  export LD_LIBRARY_PATH
fi

# chk for the required softlinks if root
if [ -n "$ROOTUSER" ]; then
  #d91010. Delete the two kgbclient links to make sure kgbconfig.sh is called every time. Otherwise, Agent may not work well after upgrading
  if [ -f $LOTUSBASEDIR/bin/kgbclient ]; then
    rm -f $LOTUSBASEDIR/bin/kgbclient
  fi
  if [ -f $Notes_ExecDirectory/kgbclient ]; then
    rm -f $Notes_ExecDirectory/kgbclient
  fi
  # chk to make sure the agent has been configured
  if [ ! -f $LOTUSBASEDIR/bin/kgbclient -o ! -f $Notes_ExecDirectory/kgbclient ]; then
    echo "`date` : Adding softlinks for $KGB_INSTANCE_NAME" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    $CANDLEHOME/$binArch/gb/bin/kgbconfig.sh $KGB_INSTANCE_NAME kgbstart >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  fi
fi

# remove the status file
if [ -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out ]; then
  echo "`date` : removing $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  rm -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
fi

#Judge whether it's a remote monitoring, if yes, no need to wait for local serer launched.
#Also for recycle.
LOCAL_MONITOR=1
DominoPID=0
# search ServerName=tivp6clp4/ibm
svr_name=`awk -F= '{if ($1=="ServerName") print $2}' $KGB_NOTESINIPATH/notes.ini`
echo "`date` : Monitoring server:$KGB_SERVER, server in notes.ini:$svr_name" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
if [ ! "$svr_name" = "$KGB_SERVER" ]; then
    short_name=`echo $svr_name|awk -F/ '{print $1}'`
    echo "`date` : short name:$short_name." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    is_localip $KGB_SERVER
    if [ $? -eq 0 ]; then
        LOCAL_MONITOR=1
    elif [ "$short_name" = "$KGB_SERVER" ]; then
        LOCAL_MONITOR=1
    else
        LOCAL_MONITOR=0
    fi
fi

if [ $LOCAL_MONITOR = 1 ]; then
    echo "`date` : Local monitoring." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
else
    echo "`date` : Remote monitoring." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
fi

#Determine whether the loop need to continute or not.
KGB_CONTE=1
while [ $KGB_CONTE = 1 ];
do
    KGB_CONTE=0
    if [ $LOCAL_MONITOR = 1 ]; then
        echo "`date` : Start waiting for domino server launched, checking pid.nbf, it should under $KGB_NOTESINIPATH directory" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        CNT=1
        MAXCNT=60
        
        # Wait until Domino Sever started, otherwise the logasio launched by kgbclient 
        # may cause server crashed.
        if [ ! -f $KGB_NOTESINIPATH/pid.nbf ]; then
          echo "`date` : Not find the file pid.nbf, it should under ${LOTUSBASEDIR}/notesdata directory, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
          exit
        fi

        while true
        do
            record=`cat  $KGB_NOTESINIPATH/pid.nbf |grep server`
            if [ $CNT -gt $MAXCNT ]; then
                echo "`date` : Warning! The domino server isn't started for 1 minute: $record" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
                CNT=0
            fi
            #detect kgbagent is running or not.
            kgbagent_running $KGBAGENT_PID
            if [ $? -eq 1 ]; then
                echo "`date` : Warning! The kgbagent has exited!exit the script now" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
                exit
            fi
            if [ "$record" = "" ]; then
                sleep 1
                CNT=`expr $CNT + 1`
                continue
            else
                svr_status=`echo $record|awk '$4=="server" {print $1}'`
        #       echo $svr_status
                if [ $svr_status -eq 1 ]; then
                    DominoPID=`echo $record|awk '$4=="server" {print $2}'`
                    break
                fi
                sleep 1
                CNT=`expr $CNT + 1`
            fi
        done
        echo "`date` : The domino server is started after $CNT seconds" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        # set the status of server to be responding and cluster to be not in
        if [ -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out ]; then
            echo 0 > $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out 
            echo 2 >> $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
            echo "`date` : Domino server status updated to *status.out" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        fi
    else
        echo "`date` : Remote monitoring. Start agent now" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    fi


    # chk for Domino Sever shutdown(1) and restart the client
    # when the Domino server process ends
    CNT=1
    MAXCNT=150
    while true
    do
      # start the Domino agent, switch to the notes user if root
      cd "$KGB_NOTESINIPATH"
      if [ -z "$ROOTUSER" ]; then
        echo "`date` : starting $LOTUSBASEDIR/bin/kgbclient as the $LOTUSOWNER user" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        echo "`date` :Root---PATH:$PATH, LIBPATH:$LIBPATH, LD_LIBRARY_PATH:$LD_LIBRARY_PATH" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        $LOTUSBASEDIR/bin/kgbclient $KGB_INSTANCE_NAME
        CR=$?
      else
        echo "`date` : Switching to the $LOTUSOWNER user and starting $LOTUSBASEDIR/bin/kgbclient" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        echo "`date` :PATH:$PATH, LIBPATH:$LIBPATH, LD_LIBRARY_PATH:$LD_LIBRARY_PATH" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        # LD_LIBRARY_PATH must be passed on Solaris. Otherwise, kgbclient can not load GSK libs.
        if [[ "$binArch" = sol+([0-9]) ]]
        then
            su $LOTUSOWNER -c "LD_LIBRARY_PATH=$LD_LIBRARY_PATH $LOTUSBASEDIR/bin/kgbclient $KGB_INSTANCE_NAME"
        else
            su $LOTUSOWNER -c "$LOTUSBASEDIR/bin/kgbclient $KGB_INSTANCE_NAME"
        fi
        CR=$?
      fi
      echo "`date` : kgbclient exited with return code(${CR})" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
      # if return code is 31, server is shutting down
      if [ $CR -eq 31 ]; then
        # set the status of server and cluster to recycling
        echo 4 > $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out 
        echo 5 >> $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
        if [ $LOCAL_MONITOR = 1 ]; then
            echo "`date` : Starting infinite waiting for server process exit: $Notes_ExecDirectory/server" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
            while true
            do
              #detect kgbagent is running or not.
              kgbagent_running $KGBAGENT_PID
              if [ $? -eq 1 ]; then
                echo "`date` : Warning! The kgbagent has exited!exit the script now" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
                exit
              fi

              # chk if server process is running
              # The $Notes_ExecDirectory could be a symbolic link, not the actual path
              # Change from search $LOTUSOWNER $Notes_ExecDirectory/server to 
              # $LOTUSOWNER server
               #not use this for it may cause alternative with name check
               #process_running $LOTUSOWNER $Notes_ExecDirectory/server
          procid_running $DominoPID
              if [ $? -eq 1 ]; then
                break
              fi
              echo "`date` : timeout count=$CNT maxcount=$MAXCNT" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
              CNT=`expr $CNT + 1`
              echo "`date` : sleeping" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
              sleep 2
            done
        else 
            echo "`date` : Remote monitoring,just start" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
            KGB_CONTE=1
            break
        fi
        CNT=1
        # kgbclient waiting for domino server start,just break to continue loop to use pid.nbf check.
        # set the server and cluster status to be not responding.
        echo 1 > $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out 
        echo 2 >> $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
        echo "`date` : kgbclient waiting for domino server start" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log

        # sleep some more to make sure
        sleep 10
        #detect kgbagent is running or not.
        kgbagent_running $KGBAGENT_PID
        if [ $? -eq 1 ]; then
            echo "`date` : Warning! The kgbagent has exited!exit the script now" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
            exit
        fi

        echo "`date` : restarting kgbclient" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        echo "=============ps -u $LOTUSOWNER -o %a|grep server=============" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        echo "`ps -u $LOTUSOWNER -o "%a"|grep server`" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        echo "==============================================================" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        
        KGB_CONTE=1
        break
      else
        KGB_CONTE=0
        if [ $CR -ne 30 ]; then
          # kill the kgbagent as kgbclient went down unexpectly
          echo "`date` : killing the kgbagent process($KGBAGENT_PID) for $KGB_INSTANCE_NAME instance..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
          kill -15 $KGBAGENT_PID
        fi
        # break and exit
        break
      fi
    done
done
# remove the status file
if [ -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out ]; then
  echo "`date` : removing $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  rm -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
fi

echo "`date` : exiting" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
